"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.compileQuery = exports.compileAST = exports.simplifyPerms = exports.mediaConditionToPerms = exports.mediaFeatureToPerms = exports.invertPerm = exports.notPerms = exports.mergePerms = exports.andPerms = void 0;
const media_query_parser_1 = require("media-query-parser");
const helpers_1 = require("./helpers");
const units_1 = require("./units");
const andPerms = (as, bs) => {
    const newPerms = [];
    for (const a of as) {
        for (const b of bs) {
            const newPerm = (0, exports.mergePerms)(a, b);
            if (Object.keys(newPerm).length > 0) {
                newPerms.push(newPerm);
            }
        }
    }
    return newPerms;
};
exports.andPerms = andPerms;
const mergePerms = (a, b) => {
    const merged = {};
    for (const p of (0, helpers_1.permToConditionPairs)(a)) {
        if (p[1] !== undefined) {
            (0, helpers_1.attachPair)(merged, p);
        }
    }
    for (const p of (0, helpers_1.permToConditionPairs)(b)) {
        if (p[0] in merged) {
            if (merged[p[0]] !== undefined) {
                // q is type safe from the above checks
                const q = merged;
                // in both, merge
                if (p[0] === "media-type") {
                    // this function doesn't merge media-types
                }
                else if (p[0] === "invalid-features") {
                    q[p[0]].push(...p[1]);
                }
                else if (q[p[0]] === "{false}" || p[1] === "{false}") {
                    q[p[0]] = "{false}";
                }
                else if (q[p[0]] === "{true}") {
                    (0, helpers_1.attachPair)(q, p);
                }
                else if (p[1] === "{true}") {
                    // nothing, q[p[0]] doesn't change
                }
                else {
                    // qq is type safe from the above checks
                    const qq = merged;
                    if ((0, helpers_1.hasRangeNumberKey)(p)) {
                        (0, helpers_1.attachPair)(qq, [p[0], (0, helpers_1.andRanges)(qq[p[0]], p[1])]);
                    }
                    else if ((0, helpers_1.hasRangeRatioKey)(p)) {
                        (0, helpers_1.attachPair)(qq, [p[0], (0, helpers_1.andRanges)(qq[p[0]], p[1])]);
                    }
                    else if (p[0] === "color-gamut") {
                        qq[p[0]] = [
                            qq[p[0]][0] && p[1][0],
                            qq[p[0]][1] && p[1][1],
                            qq[p[0]][2] && p[1][2],
                            qq[p[0]][3] && p[1][3],
                        ];
                    }
                    else if (p[0] === "any-hover" || p[0] === "hover") {
                        (0, helpers_1.attachPair)(qq, [p[0], qq[p[0]] === p[1] ? qq[p[0]] : "{false}"]);
                    }
                    else if (p[0] === "any-pointer" || p[0] === "pointer") {
                        (0, helpers_1.attachPair)(qq, [p[0], qq[p[0]] === p[1] ? qq[p[0]] : "{false}"]);
                    }
                    else if (p[0] === "grid") {
                        (0, helpers_1.attachPair)(qq, [p[0], qq[p[0]] === p[1] ? qq[p[0]] : "{false}"]);
                    }
                    else if (p[0] === "overflow-block") {
                        (0, helpers_1.attachPair)(qq, [p[0], qq[p[0]] === p[1] ? qq[p[0]] : "{false}"]);
                    }
                    else if (p[0] === "overflow-inline") {
                        (0, helpers_1.attachPair)(qq, [p[0], qq[p[0]] === p[1] ? qq[p[0]] : "{false}"]);
                    }
                    else if (p[0] === "scan") {
                        (0, helpers_1.attachPair)(qq, [p[0], qq[p[0]] === p[1] ? qq[p[0]] : "{false}"]);
                    }
                    else {
                        (0, helpers_1.attachPair)(qq, [p[0], qq[p[0]] === p[1] ? qq[p[0]] : "{false}"]);
                    }
                }
            }
        }
        else {
            (0, helpers_1.attachPair)(merged, p);
        }
    }
    return merged;
};
exports.mergePerms = mergePerms;
const notPerms = (conditionSets) => {
    // !(a || b) = !a && !b
    return conditionSets
        .map((conditionSet) => (0, exports.invertPerm)(conditionSet))
        .reduce((a, b) => (0, exports.andPerms)(a, b));
};
exports.notPerms = notPerms;
const invertPerm = (set) => {
    // !(a && b && c) = !a || !b || !c
    const pairs = (0, helpers_1.permToConditionPairs)(set);
    const combinations = [];
    for (const p of pairs) {
        if (p[1] !== undefined) {
            let condition;
            let notConditions;
            if (p[0] === "invalid-features") {
                return [{ [p[0]]: p[1] }];
            }
            else if (p[0] === "media-type") {
                continue;
            }
            else {
                condition = p;
                if (p[1] === "{false}") {
                    notConditions = [[p[0], "{true}"]];
                }
                else if (p[1] === "{true}") {
                    notConditions = [[p[0], "{false}"]];
                }
                else if ((0, helpers_1.hasDiscreteKey)(p)) {
                    if (p[0] === "color-gamut") {
                        const c = p[1];
                        notConditions = [["color-gamut", [!c[0], !c[1], !c[2], !c[3]]]];
                    }
                    else if (p[0] === "grid") {
                        notConditions = [["grid", p[1] === 0 ? 1 : 0]];
                    }
                    else {
                        notConditions = Object.keys(helpers_1.DISCRETE_FEATURES[p[0]])
                            .filter((value) => value !== p[1])
                            .map((value) => [p[0], value]);
                    }
                }
                else if ((0, helpers_1.hasRangeRatioKey)(p)) {
                    const r = (0, helpers_1.notRatioRange)(p);
                    const ranges = r === "{false}" ? ["{false}"] : r;
                    notConditions = ranges.map((range) => [p[0], range]);
                }
                else {
                    const r = (0, helpers_1.notNumberRange)(p);
                    const ranges = r === "{false}" ? ["{false}"] : r;
                    notConditions = ranges.map((range) => [p[0], range]);
                }
            }
            combinations.push([condition, notConditions]);
        }
    }
    const invertedPerms = [];
    for (const [, notConditions] of combinations) {
        for (const notCondition of notConditions) {
            invertedPerms.push({
                [notCondition[0]]: notCondition[1],
            });
        }
    }
    return invertedPerms;
};
exports.invertPerm = invertPerm;
const mediaFeatureToPerms = (mediaFeature, unitConversions) => {
    const feature = (0, units_1.simplifyMediaFeature)(mediaFeature, unitConversions);
    const INVALID = [{ "invalid-features": [mediaFeature.feature] }];
    if (feature.type === "invalid") {
        return INVALID;
    }
    else if (feature.type === "boolean") {
        if (feature.name === "color-gamut") {
            return [{ "color-gamut": [false, true, true, true] }];
        }
        else if (feature.name === "grid") {
            return [{ grid: 1 }];
        }
        else if ((0, helpers_1.isDiscreteKey)(feature.name)) {
            return (0, exports.invertPerm)({ [feature.name]: "none" });
        }
        else if ((0, helpers_1.isRangeRatioKey)(feature.name)) {
            return [{ [feature.name]: [false, [0, 1], [Infinity, 1], true] }];
        }
        else {
            return [{ [feature.name]: [false, 0, Infinity, true] }];
        }
    }
    else if ((0, helpers_1.isDiscreteKey)(feature.name)) {
        if (feature.type === "equals") {
            const unit = feature.value;
            if (feature.name === "grid") {
                if (unit.type === "number" && (unit.value === 0 || unit.value === 1)) {
                    return [{ grid: unit.value }];
                }
            }
            else if (unit.type === "ident" &&
                unit.value in helpers_1.DISCRETE_FEATURES[feature.name]) {
                if (feature.name === "color-gamut") {
                    const index = ["srgb", "p3", "rec2020"].indexOf(unit.value);
                    if (index !== -1) {
                        return [
                            { "color-gamut": [false, index <= 0, index <= 1, index <= 2] },
                        ];
                    }
                }
                else {
                    return [{ [feature.name]: unit.value }];
                }
            }
        }
        return INVALID;
    }
    else if ((0, helpers_1.isRangeRatioKey)(feature.name)) {
        let range = null;
        if (feature.type === "equals") {
            const ratio = (0, units_1.getRatio)(feature.value);
            if (ratio !== null) {
                range = [true, ratio, ratio, true];
            }
        }
        else if (feature.type === "single") {
            const ratio = (0, units_1.getRatio)(feature.value);
            if (ratio !== null) {
                if (feature.op === "<")
                    range = [true, [-Infinity, 1], ratio, false];
                else if (feature.op === "<=")
                    range = [true, [-Infinity, 1], ratio, true];
                else if (feature.op === ">")
                    range = [false, ratio, [Infinity, 1], true];
                else
                    range = [true, ratio, [Infinity, 1], true];
            }
        }
        else if (feature.type === "double") {
            const minRatio = (0, units_1.getRatio)(feature.min);
            const maxRatio = (0, units_1.getRatio)(feature.max);
            if (minRatio !== null && maxRatio !== null) {
                range = [
                    feature.minOp === "<=",
                    minRatio,
                    maxRatio,
                    feature.maxOp === "<=",
                ];
            }
        }
        return range === null
            ? INVALID
            : [{ [feature.name]: (0, helpers_1.boundRange)([feature.name, range]) }];
    }
    else {
        let range = null;
        if (feature.type === "equals") {
            const value = (0, units_1.getValue)(feature.value, feature.name);
            if (value !== null)
                range = [true, value, value, true];
        }
        else if (feature.type === "single") {
            const value = (0, units_1.getValue)(feature.value, feature.name);
            if (value !== null) {
                if (feature.op === "<")
                    range = [true, -Infinity, value, false];
                else if (feature.op === "<=")
                    range = [true, -Infinity, value, true];
                else if (feature.op === ">")
                    range = [false, value, Infinity, true];
                else
                    range = [true, value, Infinity, true];
            }
        }
        else if (feature.type === "double") {
            const minValue = (0, units_1.getValue)(feature.min, feature.name);
            const maxValue = (0, units_1.getValue)(feature.max, feature.name);
            if (minValue !== null && maxValue !== null) {
                range = [
                    feature.minOp === "<=",
                    minValue,
                    maxValue,
                    feature.maxOp === "<=",
                ];
            }
        }
        return range === null
            ? INVALID
            : [{ [feature.name]: (0, helpers_1.boundRange)([feature.name, range]) }];
    }
};
exports.mediaFeatureToPerms = mediaFeatureToPerms;
const mediaConditionToPerms = (mediaCondition, unitConversions) => {
    const conditionSetsSets = [];
    for (const child of mediaCondition.children) {
        if ("context" in child) {
            conditionSetsSets.push((0, exports.mediaFeatureToPerms)(child, unitConversions));
        }
        else {
            conditionSetsSets.push((0, exports.mediaConditionToPerms)(child, unitConversions));
        }
    }
    if (mediaCondition.operator === "or" || mediaCondition.operator === null) {
        return conditionSetsSets.flat();
    }
    else if (mediaCondition.operator === "and") {
        return conditionSetsSets.reduce((a, b) => (0, exports.andPerms)(a, b));
    }
    else {
        return (0, exports.notPerms)(conditionSetsSets[0]);
    }
};
exports.mediaConditionToPerms = mediaConditionToPerms;
const simplifyPerms = (perms) => {
    const simplePerms = [];
    const invalidFeatures = new Set();
    const falseFeatures = new Set();
    for (const perm of perms) {
        let isUnmatchable = false;
        if (Array.isArray(perm["invalid-features"])) {
            if (perm["invalid-features"].length > 0) {
                for (const invalidFeature of perm["invalid-features"]) {
                    invalidFeatures.add(invalidFeature);
                }
                isUnmatchable = true;
            }
        }
        const simplePerm = {};
        for (const p of (0, helpers_1.permToConditionPairs)(perm)) {
            if (p[0] === "invalid-features") {
                continue;
            }
            else {
                if (p[0] === "color-gamut") {
                    // because color-gamut value can be always true or always false
                    // we should convert this to "{true}" or "{false}"
                    // so it can be processed with the rest of the keys below
                    const prev = p[1].toString();
                    if (prev === "false,false,false,false") {
                        p[1] = "{false}";
                    }
                    else if (prev === "true,true,true,true") {
                        p[1] = "{true}";
                    }
                }
                else if ((0, helpers_1.hasRangeKey)(p)) {
                    p[1] = (0, helpers_1.boundRange)(p);
                }
                if (p[1] === "{false}") {
                    falseFeatures.add(p[0]);
                    isUnmatchable = true;
                }
                else if (p[1] === "{true}") {
                    // skip. {true} is removed from output
                }
                else if (p[0] === "media-type" && p[1] === "all") {
                    // skip. media-type: all is removed from output
                }
                else {
                    (0, helpers_1.attachPair)(simplePerm, p);
                }
            }
        }
        if (!isUnmatchable) {
            simplePerms.push(simplePerm);
        }
    }
    return {
        simplePerms,
        invalidFeatures: [...invalidFeatures].sort(),
        falseFeatures: [...falseFeatures].sort(),
    };
};
exports.simplifyPerms = simplifyPerms;
const compileAST = (ast, units = {}) => {
    const unitConversions = (0, units_1.compileStaticUnitConversions)(units);
    const allConditions = [];
    for (const mediaQuery of ast) {
        const extraConditions = [];
        if (mediaQuery.mediaPrefix === "not") {
            if (mediaQuery.mediaType === "print") {
                extraConditions.push({
                    "media-type": "not-print",
                });
            }
            else if (mediaQuery.mediaType === "screen") {
                extraConditions.push({
                    "media-type": "not-screen",
                });
            }
            if (mediaQuery.mediaCondition !== null) {
                extraConditions.push(...(0, exports.notPerms)((0, exports.mediaConditionToPerms)(mediaQuery.mediaCondition, unitConversions)));
            }
        }
        else {
            if (mediaQuery.mediaCondition === null) {
                extraConditions.push({
                    "media-type": mediaQuery.mediaType,
                });
            }
            else {
                extraConditions.push(...(0, exports.mediaConditionToPerms)(mediaQuery.mediaCondition, unitConversions).map((conditionSet) => (Object.assign(Object.assign({}, conditionSet), { "media-type": mediaQuery.mediaType }))));
            }
        }
        allConditions.push(...extraConditions);
    }
    return (0, exports.simplifyPerms)(allConditions);
};
exports.compileAST = compileAST;
const compileQuery = (query, units = {}) => (0, exports.compileAST)((0, media_query_parser_1.toAST)(query), units);
exports.compileQuery = compileQuery;
//# sourceMappingURL=compile.js.map