"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const compile_1 = require("./compile");
test("evaluateQuery", () => {
    expect((0, compile_1.compileQuery)("(min-width: 120px)")).toEqual((0, compile_1.simplifyPerms)([
        {
            "media-type": "all",
            width: [true, 120, Infinity, true],
        },
    ]));
    expect((0, compile_1.compileQuery)("(min-width: 100px) and (max-width: 200px)")).toEqual((0, compile_1.simplifyPerms)([
        {
            "media-type": "all",
            width: [true, 100, 200, true],
        },
    ]));
    expect((0, compile_1.compileQuery)("(50px < width <= 150px)")).toEqual((0, compile_1.simplifyPerms)([
        {
            "media-type": "all",
            width: [false, 50, 150, true],
        },
    ]));
});
test("checking hyphenated keys", () => {
    expect((0, compile_1.compileQuery)("(min-aspect-ratio: 1/2)")).toEqual({
        invalidFeatures: [],
        falseFeatures: [],
        simplePerms: [
            {
                "aspect-ratio": [true, [1, 2], [Infinity, 1], false],
            },
        ],
    });
});
test("handles not queries", () => {
    expect((0, compile_1.compileQuery)("not (min-width: 120px)")).toEqual((0, compile_1.simplifyPerms)([
        {
            width: [true, 0, 120, false],
        },
    ]));
    expect((0, compile_1.compileQuery)("not ((min-width: 100px) and (max-width: 200px))")).toEqual((0, compile_1.simplifyPerms)([
        {
            width: [true, 0, 100, false],
        },
        {
            width: [false, 200, Infinity, true],
        },
    ]));
    expect((0, compile_1.compileQuery)("screen and (not ((min-width: 100px) and (max-width: 200px)))")).toEqual((0, compile_1.simplifyPerms)([
        {
            "media-type": "screen",
            width: [true, 0, 100, false],
        },
        {
            "media-type": "screen",
            width: [false, 200, Infinity, true],
        },
    ]));
    expect((0, compile_1.compileQuery)("not print and (min-width: 100px) and (max-width: 200px)")).toEqual((0, compile_1.simplifyPerms)([
        {
            "media-type": "not-print",
        },
        {
            width: [true, 0, 100, false],
        },
        {
            width: [false, 200, Infinity, true],
        },
    ]));
    expect((0, compile_1.compileQuery)("not screen and (not ((min-width: 100px) and (max-width: 200px)))")).toEqual((0, compile_1.simplifyPerms)([
        {
            "media-type": "not-screen",
        },
        {
            width: [true, 100, 200, true],
        },
    ]));
    expect((0, compile_1.compileQuery)("not ((min-width: 100px) and (max-width: 200px))")).toEqual({
        invalidFeatures: [],
        falseFeatures: [],
        simplePerms: [
            { width: [true, 0, 100, false] },
            { width: [false, 200, Infinity, false] },
        ],
    });
    expect(() => (0, compile_1.compileQuery)("not (min-width: 100px) and (max-width: 200px)")).toThrow();
});
test("correctly handles weird queries", () => {
    expect((0, compile_1.compileQuery)("(fake: feature)")).toEqual({
        invalidFeatures: ["fake"],
        falseFeatures: [],
        simplePerms: [],
    });
    expect((0, compile_1.compileQuery)("not (width: infinite)")).toEqual({
        invalidFeatures: ["width"],
        falseFeatures: [],
        simplePerms: [],
    });
    expect((0, compile_1.compileQuery)("(aspect-ratio: 0.01/1)")).toEqual((0, compile_1.simplifyPerms)([
        {
            "aspect-ratio": [true, [0.01, 1], [0.01, 1], true],
        },
    ]));
    expect((0, compile_1.compileQuery)("(height < 100px) or (height > 50px)")).toEqual((0, compile_1.simplifyPerms)([
        {
            height: [true, 0, 100, false],
        },
        {
            height: [false, 50, Infinity, false],
        },
    ]));
});
test("not operator", () => {
    expect((0, compile_1.compileQuery)("not (width < -1px)")).toEqual((0, compile_1.simplifyPerms)([{}]));
    expect((0, compile_1.compileQuery)("not (min-width: 120px)")).toEqual((0, compile_1.simplifyPerms)([
        {
            "media-type": "all",
            width: [true, 0, 120, false],
        },
    ]));
    expect((0, compile_1.compileQuery)("not (max-width: 240px)")).toEqual((0, compile_1.simplifyPerms)([
        {
            "media-type": "all",
            width: [false, 240, Infinity, true],
        },
    ]));
    expect((0, compile_1.compileQuery)("not (110px <= width <= 220px)")).toEqual((0, compile_1.simplifyPerms)([
        {
            "media-type": "all",
            width: [true, 0, 110, false],
        },
        {
            "media-type": "all",
            width: [false, 220, Infinity, true],
        },
    ]));
    expect((0, compile_1.compileQuery)("screen and (not (min-width: 120px))")).toEqual((0, compile_1.simplifyPerms)([
        {
            "media-type": "screen",
            width: [true, 0, 120, false],
        },
    ]));
    expect((0, compile_1.compileQuery)("print and (not (max-width: 240px))")).toEqual((0, compile_1.simplifyPerms)([
        {
            "media-type": "print",
            width: [false, 240, Infinity, true],
        },
    ]));
    expect((0, compile_1.compileQuery)("not print and (110px <= width <= 220px)")).toEqual((0, compile_1.simplifyPerms)([
        {
            "media-type": "not-print",
        },
        {
            width: [true, 0, 110, false],
        },
        {
            width: [false, 220, Infinity, false],
        },
    ]));
    expect((0, compile_1.compileQuery)("not print and (not (110px <= width <= 220px))")).toEqual((0, compile_1.simplifyPerms)([
        {
            "media-type": "not-print",
        },
        {
            width: [true, 110, 220, true],
        },
    ]));
});
test("custom units", () => {
    expect((0, compile_1.compileQuery)("(width: 1em)", {
        emPx: 1280,
    })).toEqual((0, compile_1.simplifyPerms)([
        {
            width: [true, 1280, 1280, true],
        },
    ]));
});
test("found bugs", () => {
    expect((0, compile_1.compileQuery)("not screen and (((not ((min-width: 1000px) and (orientation: landscape))) or (color))), (monochrome)")).toEqual((0, compile_1.simplifyPerms)([
        {
            "media-type": "not-screen",
        },
        {
            "aspect-ratio": [true, [1, 1], [Infinity, 1], false],
            color: [true, 0, 0, true],
            width: [true, 1000, Infinity, false],
        },
        {
            monochrome: [false, 0, Infinity, false],
        },
    ]));
    expect((0, compile_1.compileQuery)("(16/10 <= aspect-ratio < 16/10)")).toEqual({
        falseFeatures: ["aspect-ratio"],
        invalidFeatures: [],
        simplePerms: [],
    });
    expect((0, compile_1.compileQuery)("(hover)")).toEqual({
        falseFeatures: [],
        invalidFeatures: [],
        simplePerms: [
            {
                hover: "hover",
            },
        ],
    });
    expect((0, compile_1.simplifyPerms)((0, compile_1.invertPerm)({
        width: [true, 1000, Infinity, true],
        "aspect-ratio": [true, [1, 1], [Infinity, 1], false],
    }))).toEqual({
        falseFeatures: [],
        invalidFeatures: [],
        simplePerms: [
            {
                width: [true, 0, 1000, false],
            },
            {
                "aspect-ratio": [false, [0, 1], [1, 1], false],
            },
        ],
    });
    expect((0, compile_1.compileQuery)("(orientation)")).toEqual({
        falseFeatures: [],
        invalidFeatures: [],
        simplePerms: [{}],
    });
    expect((0, compile_1.compileQuery)("not screen and (min-width: 1000px) and (orientation: landscape)")).toEqual({
        falseFeatures: [],
        invalidFeatures: [],
        simplePerms: [
            { "media-type": "not-screen" },
            {
                width: [true, 0, 1000, false],
            },
            {
                "aspect-ratio": [false, [0, 1], [1, 1], false],
            },
        ],
    });
    expect((0, compile_1.compileQuery)("(not ((min-width: 1000px) and (orientation: landscape)))")).toEqual({
        simplePerms: [
            {
                width: [true, 0, 1000, false],
            },
            {
                "aspect-ratio": [false, [0, 1], [1, 1], false],
            },
        ],
        invalidFeatures: [],
        falseFeatures: [],
    });
});
//# sourceMappingURL=compile.test.js.map