export declare const log: (x: unknown) => void;
declare type Values<T> = T[keyof T];
declare type KVPairs<T> = Values<{
    [Property in keyof T]: [Property, T[Property]];
}>;
export declare type ConditionRange<T = number> = readonly [boolean, T, T, boolean];
export declare type Condition<T> = T | "{false}" | "{true}";
export declare type Conditional<T> = {
    [Property in keyof T]: Condition<T[Property]>;
};
export declare type DiscreteFeatures = {
    "any-hover": "none" | "hover";
    "any-pointer": "none" | "coarse" | "fine";
    "color-gamut": readonly [
        belowSrgb: boolean,
        srgbAndBelowP3: boolean,
        p3AndBelowRec2020: boolean,
        rec2020AndAbove: boolean
    ];
    grid: 0 | 1;
    hover: "none" | "hover";
    "overflow-block": "none" | "scroll" | "paged";
    "overflow-inline": "none" | "scroll";
    pointer: "none" | "coarse" | "fine";
    scan: "interlace" | "progressive";
    update: "none" | "slow" | "fast";
};
export declare type RangeRatioFeatures = {
    "aspect-ratio": ConditionRange<readonly [number, number]>;
    "device-aspect-ratio": ConditionRange<readonly [number, number]>;
};
export declare type RangeNumberFeatures = {
    color: ConditionRange;
    "color-index": ConditionRange;
    "device-height": ConditionRange;
    "device-width": ConditionRange;
    height: ConditionRange;
    monochrome: ConditionRange;
    resolution: ConditionRange;
    width: ConditionRange;
};
export declare type RangeFeatures = RangeRatioFeatures & RangeNumberFeatures;
export declare type MediaFeatures = DiscreteFeatures & RangeFeatures;
export declare type Perm = Partial<Conditional<MediaFeatures> & {
    "media-type": "screen" | "print" | "not-screen" | "not-print" | "all" | "{false}";
    "invalid-features": string[];
}>;
export declare type ConditionPair = KVPairs<Required<Perm>>;
export declare type DiscreteConditionPair = KVPairs<Conditional<DiscreteFeatures>>;
export declare type RangeConditionPair = KVPairs<Conditional<RangeFeatures>>;
export declare type RangeRatioConditionPair = KVPairs<Conditional<RangeRatioFeatures>>;
export declare type RangeNumberConditionPair = KVPairs<Conditional<RangeNumberFeatures>>;
export declare const permToConditionPairs: (perm: Perm) => ConditionPair[];
export declare const hasDiscreteKey: (pair: ConditionPair) => pair is DiscreteConditionPair;
export declare const isDiscreteKey: (key: string) => key is keyof DiscreteFeatures;
export declare const hasRangeRatioKey: (pair: ConditionPair) => pair is RangeRatioConditionPair;
export declare const isRangeRatioKey: (key: string) => key is keyof RangeRatioFeatures;
export declare const hasRangeNumberKey: (pair: ConditionPair) => pair is RangeNumberConditionPair;
export declare const isRangeNumberKey: (key: string) => key is keyof RangeNumberFeatures;
export declare const hasRangeKey: (pair: ConditionPair) => pair is RangeConditionPair;
export declare const isRangeKey: (key: string) => key is keyof RangeNumberFeatures;
export declare const isPermKey: (key: string) => key is keyof DiscreteFeatures | keyof RangeRatioFeatures | keyof RangeNumberFeatures | "media-type" | "invalid-features";
export declare const isFeatureKey: (key: string) => key is keyof DiscreteFeatures | keyof RangeRatioFeatures | keyof RangeNumberFeatures;
export declare const DISCRETE_FEATURES: {
    readonly "any-hover": {
        readonly none: 1;
        readonly hover: 1;
    };
    readonly "any-pointer": {
        readonly none: 1;
        readonly coarse: 1;
        readonly fine: 1;
    };
    readonly "color-gamut": {
        readonly srgb: 1;
        readonly p3: 1;
        readonly rec2020: 1;
    };
    readonly grid: {
        readonly 0: 1;
        readonly 1: 1;
    };
    readonly hover: {
        readonly none: 1;
        readonly hover: 1;
    };
    readonly "overflow-block": {
        readonly none: 1;
        readonly scroll: 1;
        readonly paged: 1;
    };
    readonly "overflow-inline": {
        readonly none: 1;
        readonly scroll: 1;
    };
    readonly pointer: {
        readonly none: 1;
        readonly coarse: 1;
        readonly fine: 1;
    };
    readonly scan: {
        readonly interlace: 1;
        readonly progressive: 1;
    };
    readonly update: {
        readonly none: 1;
        readonly slow: 1;
        readonly fast: 1;
    };
};
export declare const RANGE_NUMBER_FEATURES: {
    readonly color: {
        readonly feature: "color";
        readonly type: "integer";
        readonly bounds: readonly [true, 0, number, false];
    };
    readonly "color-index": {
        readonly feature: "color-index";
        readonly type: "integer";
        readonly bounds: readonly [true, 0, number, false];
    };
    readonly monochrome: {
        readonly feature: "monochrome";
        readonly type: "integer";
        readonly bounds: readonly [true, 0, number, false];
    };
    readonly "device-height": {
        readonly feature: "device-height";
        readonly type: "length";
        readonly bounds: readonly [true, 0, number, false];
    };
    readonly "device-width": {
        readonly feature: "device-width";
        readonly type: "length";
        readonly bounds: readonly [true, 0, number, false];
    };
    readonly height: {
        readonly feature: "height";
        readonly type: "length";
        readonly bounds: readonly [true, 0, number, false];
    };
    readonly width: {
        readonly feature: "width";
        readonly type: "length";
        readonly bounds: readonly [true, 0, number, false];
    };
    readonly resolution: {
        readonly feature: "resolution";
        readonly type: "resolution";
        readonly bounds: readonly [true, 0, number, false];
    };
};
export declare const RANGE_RATIO_FEATURES: {
    readonly "aspect-ratio": {
        readonly feature: "aspect-ratio";
        readonly type: "ratio";
        readonly bounds: readonly [false, readonly [0, 1], readonly [number, 1], false];
    };
    readonly "device-aspect-ratio": {
        readonly feature: "device-aspect-ratio";
        readonly type: "ratio";
        readonly bounds: readonly [false, readonly [0, 1], readonly [number, 1], false];
    };
};
export declare const attachPair: <T extends object>(obj: T, pair: ConditionPair) => void;
export declare const andRanges: <T extends number | readonly [number, number]>(...ranges: ("{false}" | "{true}" | ConditionRange<T>)[]) => "{false}" | "{true}" | ConditionRange<T>;
export declare const boundRange: <T extends RangeConditionPair>(pair: T) => T[1];
export declare const binaryOrRanges: <T extends number | readonly [number, number]>(a: ConditionRange<T>, b: ConditionRange<T>) => ConditionRange<T>[];
export declare const notRatioRange: (pair: RangeRatioConditionPair) => ConditionRange<readonly [number, number]>[] | "{false}";
export declare function notNumberRange(pair: RangeNumberConditionPair): ConditionRange[] | "{false}";
export {};
//# sourceMappingURL=helpers.d.ts.map